//Modelo Dinamico de cuadricoptero

function ydot = DynMod(t,y,delta)
   // variables de control
//delta(1); // forward motor
//delta(2); // right motor
//delta(3); // back motor
//delta(4); // left motor
  forces=P.k_1*delta(:).^2
  torques=P.k_2*delta(:).^2
  
  Vel=y(4:6); // velocidades en body frame
  
  phi=y(7); // Roll angle defined with respect to F^v2
  theta=y(8); // Pitch angle defined with respect to F^v1
  psi=y(9); // Heading (yaw) angle defined with respect to F^v
  V_a=norm(Vel,2);
  alpha=atan(y(6),y(4));
  Beta=asin(y(5)/V_a);
  VelAng=y(10:12); // [p,q,r] velocidades angulares
  p=y(10); q=y(11); r=y(12);
  
  Sphi=sin(phi); Cphi=cos(phi);
  Stheta=sin(theta); Ctheta=cos(theta);
  Spsi=sin(psi); Cpsi=cos(psi);
  Ttheta=tan(theta);
  Calpha=cos(alpha); Salpha=sin(alpha);
  Cbeta=cos(Beta); Sbeta=sin(Beta);
  
  Rvtov1=[Cpsi,Spsi,0;-Spsi,Cpsi,0;0,0,1];
  Rv1tov2=[Ctheta,0,-Stheta;0,1,0;Stheta,0,Ctheta];
  Rv2tob=[1,0,0;0,Cphi,Sphi;0,-Sphi,Cphi];
  Rvtob=Rv2tob*Rv1tov2*Rvtov1;
  Rbtos=[Calpha,0,Salpha;0,1,0;-Salpha,0,Calpha]; 
  Rstob=Rbtos';
  R33=[1,Sphi*Ttheta,Cphi*Ttheta;
       0,Cphi,-Sphi;
       0,Sphi/Ctheta,Cphi/Ctheta];
       
  // fuerzas 
  f_g=Rvtob*[0;0;P.m*P.g];
  f_p=[0; 0; -sum(forces(:))];  
//  F_drag=Coeff*(C_D+P.C_D_q*P.c*q/(2*V_a)+P.C_D_delta_e*delta_e);
//
//  f_y=Coeff*(P.C_Y_0+P.C_Y_Beta*Beta+P.C_Y_p*P.b*p/(2*V_a)+...
//      P.C_Y_r*P.b*r/(2*V_a)+P.C_Y_delta_a*delta_a+P.C_Y_delta_r*delta_r);
//  F_lift=Coeff*(C_L+P.C_L_q*P.c*q/(2*V_a)+P.C_L_delta_e*delta_e);
//  f_a=Rstob*[-F_drag;f_y;-F_lift];
  f_a=[0;0;0]  
  fuerzas=f_g + f_a + f_p;
  
//  Momentos
  tau_phi=P.L*(forces(4)-forces(2))
  tau_theta=P.L*(forces(1)-forces(3))
  tau_psi=torques(2)+torques(4)-torques(1)-torques(3)
  momentos=[tau_phi;tau_theta;tau_psi]
  
  // dinámica
  dotPos=Rvtob'*Vel;
  dotVel=fuerzas/P.m-cross(VelAng,Vel);
  dotAng=R33*VelAng;
  dotVelAng=inv(P.J)*(momentos-cross(VelAng,(P.J*VelAng)))

  ydot=[dotPos;dotVel;dotAng;dotVelAng];
endfunction

function y =DynamicModel(y0, tInit, t, delta)
  y = ode(y0, tInit, t, list(DynMod,delta));
endfunction
