// SensParam.sce

//*****************************************************************************
// Programa de evaluación y pruebas aleatorias automatizadas de
// los algoritmos de estimación de orientación para vehículos aéreos.
// Autores: Yuriy Kovalenko, Gilberto Bustamante Balderas, Pedro Huerta Durán
//*****************************************************************************

//Analog Devices ADXL325 MEMS accelerometer. 
//range of ±6g,
//variable bandwidth of 0.5 to 550 Hz.
Ba=100; //For an autopilot, a typical bandwidth is 100 Hz.
Na=250e-6; //noise density of 250 μg/Hz^0.5

//standard deviation of the measurement error due to sensor noise
sigma_accel=Na*sqrt(Ba);
sigma_accel_x=sigma_accel;
sigma_accel_y=sigma_accel;
sigma_accel_z=sigma_accel;

//Analog Devices ADXRS450 MEMS rate gyro
//range of ±350 deg/s, a bandwidth of 80 Hz
Bg=80; //Hz
//noise density of 0.015 deg/s/Hz.
Ng=0.015;

//standard deviation of the measurement error due to sensor noise
sigma_gyro=Ng*sqrt(Bg)*%pi/180; //rad/s
sigma_gyro_x=sigma_gyro;
sigma_gyro_y=sigma_gyro;
sigma_gyro_z=sigma_gyro;

// absolute pressure sensor for altitude measurement
// Freescale Semiconductor MP3H6115A. Range of 15 to 115 kPa,
// and its maximum error is a 1.5 percent of the full scale, or ±1.5 kPa. 
// The book says we could use about 0.125 kPa of temperature-related 
// bias drift and about 0.01 kPa of sensor noise.
beta_abs_pres = 0.125;// kPa 
sigma_abs_pres = 0.01;// kPa

// differential pressure sensor for use as an airspeed sensor.
// Freescale Semiconductor MPXV5004G. Range of 0 to 4 kPa, 
//  maximum error has a bound of 2.5 percent of the full-scale range, or 0.1 kPa.
// The book says we could use about 0.02 kPa of temperature-related bias drift 
// and about 0.002 kPa of sensor noise.
beta_diff_pres = 0.020;// kPa
sigma_diff_pres = 0.002;// kPa

// GPS
kGPS=1/1100; //1/kGPS is the time constant of the Markov process, 
TsGPS = 1.0; //(sec) sample time.

sigma_GPS_pn=0.21; // Std. Dev. ηGPS (m) pos. north
sigma_GPS_pe=0.21; // Std. Dev. ηGPS (m) pos. east
sigma_GPS_pd=0.40; // Std. Dev. ηGPS (m) pos. down

