// Sensors.sce

//*****************************************************************************
// Programa de evaluación y pruebas aleatorias automatizadas de
// los algoritmos de estimación de orientación para vehículos aéreos.
// Autores: Yuriy Kovalenko, Gilberto Bustamante Balderas, Pedro Huerta Durán
//*****************************************************************************

  
function SensOut=sensors(state)
  global delta
  pd=state(3);//position down
  Vel=state(4:6); // velocidades en body frame
  phi=state(7);//roll_ang
  theta=state(8);//pitch_ang
//  psi=state(:,9);//yaw_ang
  VelAng=state(10:12); // [p,q,r] velocidades angulares

  //zero-mean Gaussian processes
  //Y=grand(m, n, "nor", Av, Sd) generates random variates
  //from the normal distribution with mean Av and standard deviation Sd.

  eta_accel_x=NoiseLevel*grand(1,1,"nor",0,sigma_accel_x);
  eta_accel_y=NoiseLevel*grand(1,1,"nor",0,sigma_accel_y);
  eta_accel_z=NoiseLevel*grand(1,1,"nor",0,sigma_accel_z);

  eta_gyro_x=NoiseLevel*grand(1,1,"nor",0,sigma_gyro_x);
  eta_gyro_y=NoiseLevel*grand(1,1,"nor",0,sigma_gyro_y); 
  eta_gyro_z=NoiseLevel*grand(1,1,"nor",0,sigma_gyro_z);
  
  eta_abs_pres=NoiseLevel*grand(1,1,"nor",0,sigma_abs_pres);
  eta_diff_pres=NoiseLevel*grand(1,1,"nor",0,sigma_diff_pres);
 
  //acceleration components. equ. 7.3
  ydot = DynMod(0,state,delta);
  dot_vel=ydot(4:6);// dot_u; dot_v; dot_w;
//  GravAccel=P.g*[-sin(theta); sin(phi)*cos(theta); cos(phi)*cos(theta)];
  GravAccel=0;
  y_accel=dot_vel+cross(VelAng,Vel)+GravAccel//+...
          [eta_accel_x;eta_accel_y;eta_accel_z];

  //gyro signal components equ. 7.5
  y_gyro = VelAng+[eta_gyro_x;eta_gyro_y;eta_gyro_z];
  
  //Pressure sensors
  //Absolute pressure (reference to sealed vacuum) in kPa
  hAGL=-pd;
  y_abs_pres=P.Pstd+P.rho*P.g*hAGL+beta_abs_pres+eta_abs_pres;//equ. 7.9
  
  Va=norm(Vel,2);
//Va is airspeed of the MAV
  y_diff_pres=P.rho*Va^2/2+beta_diff_pres+eta_diff_pres; //equ. 7.10
  
  SensOut=[y_accel;y_gyro;y_abs_pres;y_diff_pres];
endfunction

function GpsOut=GPS(pn,pe,pd)  //GPS sensors simulation
  global nu_pn nu_pe nu_pd
  
// a model the GPS estimation error as a Gauss-Markov process
  etaGPS_pn=grand(1,1,"nor",0,sigma_GPS_pn);// zero-mean Gaussian white noise
  etaGPS_pe=grand(1,1,"nor",0,sigma_GPS_pe);// zero-mean Gaussian white noise
  etaGPS_pd=grand(1,1,"nor",0,sigma_GPS_pd);// zero-mean Gaussian white noise
  
  nu_pn=exp(-kGPS*TsGPS)*nu_pn+etaGPS_pn; //equ. (7.17)
  nu_pe=exp(-kGPS*TsGPS)*nu_pe+etaGPS_pe; //equ. (7.17)
  nu_pd=exp(-kGPS*TsGPS)*nu_pd+etaGPS_pd; //equ. (7.17)
  
  y_GPS_pn=pn+NoiseLevel*nu_pn; // equ.(7.18)
  y_GPS_pe=pe+NoiseLevel*nu_pe; // equ.(7.19)
  y_GPS_h =-pd+NoiseLevel*nu_pd; // equ.(7.20)
//  y_GPS_Vg=sqrt((Va*cos(Psi)+w_n)^2+(Va*sin(Psi)+w_e)^2)+etaV;//equ. (7.25)

  GpsOut=[y_GPS_pn;y_GPS_pe;y_GPS_h];
endfunction

function SensResults=SensorSimul()
  global nu_pn nu_pe nu_pd
 
  nu_pn=0; nu_pe=0; nu_pd=0;
   
  n=length(t);
  SensResults=zeros(11,n)
  NextGpsSample=0;
  for i=1:n
   if t(i)>=NextGpsSample
    GpsOut=GPS(MDResult(1,i),MDResult(2,i),MDResult(3,i));//pn, pe, pd
    NextGpsSample=NextGpsSample+TsGPS;
   end  
    SensResults(:,i)=[sensors(MDResult(:,i));GpsOut];
  end
endfunction
