// trim.sce

//*****************************************************************************
// Programa de evaluación y pruebas aleatorias automatizadas de
// los algoritmos de estimación de orientación para vehículos aéreos.
// Autores: Yuriy Kovalenko, Gilberto Bustamante Balderas, Pedro Huerta Durán
//*****************************************************************************

function z=ObjFunct(x_u)
 global xDot
 global R
 if(R==0 | R==%inf)
  x_t=[0;0;0; x_u(1);0;x_u(2); 0;x_u(3);0; 0;0;0];
  u_t=[0;x_u(4);0;x_u(5)];
 else
  x_t=[0;0;0; x_u(1:9)];
  u_t=x_u(10:13);
 end 
 xDotCurrent=DynMod(0,x_t,u_t);
 weight=[1;1;1;1;1;1;1;1;1;1];

 z=norm((weight.*xDot(3:$)-weight.*xDotCurrent(3:$)),2);//Euclidian norm
endfunction

function [xTrim,delta]=trim(TrimPar)
//Steady-State Flight Mode 
//Steady wings-level flight="WingsLevelFlight"
//Steady turning flight="CoordinatedTurn"

 global xDot
 global R
 Va=TrimPar(1);
 Gamma=TrimPar(2);
 R=TrimPar(3);
 
 if(R==0 | R==%inf)
  printf("\nWings Level Flight\n");
  xDot=[0;0;-Va*sin(Gamma); 0;0;0; 0;0;0; 0;0;0];
  x0=[Va;0;0];//only [u,w,theta]
  u0=[0;1];//only [delta_e,delta_t]//elevator and thrust
 else
  printf("\nCoordinated Turn\n");
  xDot=[0;0;-Va*sin(Gamma); 0;0;0; 0;0;Va*cos(Gamma)/R; 0;0;0];
  x0=[Va;0;0; 0;0;0; 0;0;0];//only [u,v,w,phi,theta,psi,p,q,r]
  u0=[0;0;0;1];//only [delta_a,delta_e,delta_t]//aileron, elevator and thrust
 end  
 
 disp(xDot',"xDot ast vector")
 x_u=[x0;u0];
 //non-linear least squares optimization 
 [fopt,xopt] = leastsq(ObjFunct,x_u);
 printf("\nTrim error = %e\n",fopt);
 
 if(R==0 | R==%inf)
  xTrim=[0;0;0; xopt(1);0;xopt(2); 0;xopt(3);0; 0;0;0];
  delta=[0;xopt(4);0;xopt(5)];
 else
  xTrim=[0;0;0; xopt(1:9)];
  delta=xopt(10:13);
 end  
endfunction


